// Modules to control application life and create native browser window
const { app, BrowserWindow, protocol, ipcMain, dialog, nativeImage } = require('electron')
const path = require('path')

const gotTheLock = app.requestSingleInstanceLock();

if (!gotTheLock) {
  app.quit();
  return;
}

let mainWindow;
function createWindow() {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 1080,
    height: 600,
    minWidth: 400,
    webPreferences: {
      nodeIntegration: true,
      // javascript: true,
      // plugins: true,
      preload: path.join(__dirname, './', 'preload.js'),
    },
    // frame: false,
  })
  // and load the index.html of the app.
  // mainWindow.loadFile('./dist/index.html')
  // mainWindow.loadURL('http://47.104.91.134:9000');
  mainWindow.loadURL('http://47.104.91.134:9000', { "extraHeaders": "pragma: no-cache\n" });
  // mainWindow.loadURL('http://localhost:8081');
  // mainWindow.loadURL('http://10.240.33.229:8081');
  // 隐藏菜单栏
  mainWindow.setMenuBarVisibility(false);
  // Open the DevTools.
  // mainWindow.webContents.openDevTools()
  mainWindow.maximize();

  // 当窗口已经关闭的时候触发
  mainWindow.on('closed', (event) => {
    mainWindow = null;
  });
  // 当窗口要关闭的时候触发
  mainWindow.on('close', (event) => { 
    mainWindow.hide(); 
    mainWindow.setSkipTaskbar(true);
    event.preventDefault();
  });

  mainWindow.on('show', () => {
    tray.setHighlightMode('always')
  })
  mainWindow.on('hide', () => {
    tray.setHighlightMode('never')
  })

  // 创建系统通知区菜单
  tray = new Tray(path.join(__dirname, 'file_word.png'));
   const contextMenu = Menu.buildFromTemplate([
      {label: '退出', click: () => {mainWindow.destroy()}},//我们需要在这里有一个真正的退出（这里直接强制退出）
    ])
    tray.setToolTip('LOCKING探索者')
    tray.setContextMenu(contextMenu);
    tray.on('click', ()=>{ //我们这里模拟桌面程序点击通知区图标实现打开关闭应用的功能
        mainWindow.isVisible() ? mainWindow.hide() : mainWindow.show()
        mainWindow.isVisible() ? mainWindow.setSkipTaskbar(false) : mainWindow.setSkipTaskbar(true);
    })
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(() => {
  createWindow()
  app.on('activate', function () {
    // On macOS it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) createWindow()
  })
})

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', function () {
  if (process.platform !== 'darwin') app.quit()
})

app.on('second-instance', (event, commandLine, workingDirectory) => {
  // 当运行第二个实例时,将会聚焦到mainWindow这个窗口
  if (mainWindow) {
    if (mainWindow.isMinimized()) mainWindow.restore()
    mainWindow.focus()
    mainWindow.show()
  }
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.

// 监听必要的自定义事件
/**
 * 项目中的文件上传
 */
ipcMain.handle('project-choose-files', async (event, args) => {
  const res = await dialog.showOpenDialog({
    properties: ['multiSelections', 'openFile'],
  });
  return res;
});
ipcMain.handle('project-choose-folders', async (event, args) => {
  const res = await dialog.showOpenDialog({
    properties: ['multiSelections', 'openDirectory'],
  });
  return res;
});
const testIcon = nativeImage.createFromPath(path.resolve(__dirname, 'file_word.png'));

ipcMain.handle('project-file-dnd', (event, filePath) => {
  console.log('receive file path: ', filePath)
  event.sender.startDrag({
    file: filePath, // path.resolve(__dirname, 'file_word.png'),
    // icon: nativeImage.createFromNamedImage('NS')
    icon: testIcon,
  });
})